#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "config.h"
#include "version.h"

static struct config* cfg;

struct config* config_init( int argc, char **argv )
{
    char opt;
    uint32 optcnt = 0;
    char* env;

    cfg = (struct config*)malloc( sizeof( struct config ) );

    cfg->hashsize = HASHSIZE;
    cfg->capture_method = CM_NONE;
    strcpy( cfg->savedir, SAVEDIR );
    strcpy( cfg->capture_device, CAPDEVICE );
    strcpy( cfg->pidfile, PIDFILE );
    cfg->max_flows_in_queue = MAXFLOWSINQUEUE;
    cfg->flows_per_file = FLOWSPERFILE;
    cfg->flow_timeout = FLOWTIMEOUT;
    cfg->bpf_size = BPFSIZE;
    cfg->bpf_timeout = BPFTIMEOUT;
    cfg->stat_delay = STATDELAY;

    if( env = getenv( "NITPICKER_CAPDEVICE" ), env ) {
        strncpy( cfg->capture_device, env, sizeof cfg->capture_device );
    }

    if( env = getenv( "NITPICKER_CAPMETHOD" ), env ) {
        if( !strcmp( env, "BPF" ) )
            cfg->capture_method = CM_BPF;
        else if( !strcmp( env, "PCAPLIVE" ) )
            cfg->capture_method = CM_PCAPLIVE;
        else if( !strcmp( env, "PCAPFILE" ) )
            cfg->capture_method = CM_PCAPFILE;
    } 


    if( env = getenv( "NITPICKER_SAVEDIR" ), env )
        strncpy( cfg->savedir, env, sizeof cfg->savedir );

    if( env = getenv( "NITPICKER_HASHSIZE" ), env )
        cfg->hashsize = atoi( env );

    if( env = getenv( "NITPICKER_MAXFLOWSINQUEUE" ), env )
        cfg->max_flows_in_queue = atoi( env );

    if( env = getenv( "NITPICKER_FLOWSPERFILE" ), env )
        cfg->flows_per_file = atoi( env );

    if( env = getenv( "NITPICKER_FLOWTIMEOUT" ), env )
        cfg->flow_timeout = atoi( env );

    if( env = getenv( "NITPICKER_BPFSIZE" ), env )
        cfg->bpf_size = atoi( env );

    if( env = getenv( "NITPICKER_BPFTIMEOUT" ), env )
        cfg->bpf_timeout = atoi( env );

    if( env = getenv( "NITPICKER_STATDELAY" ), env )
        cfg->stat_delay = atoi( env );

    if( env = getenv( "NITPICKER_PIDFILE" ), env )
        strncpy( cfg->pidfile, env, sizeof cfg->pidfile );

    while (( opt = getopt(argc, argv, "vcp:d:i:I:f:s:")) != -1 ) {
        optcnt++;
        switch( opt ) {

            case 'v': /* view config */
                config_usage();
                break;

            case 'c': /* view config */
                config_dump();
                break;
 
            case 'i': /* BPF */
                strncpy( cfg->capture_device, optarg, sizeof cfg->capture_device );
                cfg->capture_method = CM_BPF;   
                break;

            case 'I': /* PCAP Live */
                strncpy( cfg->capture_device, optarg, sizeof cfg->capture_device );
                cfg->capture_method = CM_PCAPLIVE; 
                break;

            case 'f': /* PCAP File */
                strncpy( cfg->capture_device, optarg, sizeof cfg->capture_device );
                cfg->capture_method = CM_PCAPFILE;   
                break;

            case 's':
                strncpy( cfg->savedir, optarg, sizeof cfg->savedir );
                break;
        }

        argc -= optind;
        argv += optind;

        /*
           printf( "argv0:%d\n", (int)*argv );
           printf( "argv0:%s\n", argv[0] );
           */
    }


    return cfg;
}

void config_usage()      
{
 printf("\nVersion:%s\n",Version);
 printf("Usage: nitpicker [-i <interface>|-I <interface>|-f <pcapdumpfile>] [-d <debuglevel>] [-s <flowsavedir>] [-p <pidfile>] \n\n");
 printf(" -i\tUses BPF Device to capture packets (recommended)\n");
 printf(" -I\tUses libpcap to capture packets from a network device\n");
 printf(" -f\tUses libpcap to read packets from a dumpfile (e.g. tcpdump)\n");
 printf(" -p\twrite PID-File\n");
 printf(" -c\tdumps config\n" );
 printf("\n");
}

void config_dump()
{
 switch( cfg->capture_method ) {
     case CM_BPF:
         printf( "NITPICKER_CAPMETHOD=BPF\n"
                 "# NITPICKER_CAPMETHOD=PCAPLIVE\n"
                 "# NITPICKER_CAPMETHOD=PCAPFILE\n" );
         break;

     case CM_PCAPLIVE:
         printf( "# NITPICKER_CAPMETHOD=BPF\n"
                 "NITPICKER_CAPMETHOD=PCAPLIVE\n"
                 "# NITPICKER_CAPMETHOD=PCAPFILE\n" );
         break;

     case CM_PCAPFILE:
         printf( "# NITPICKER_CAPMETHOD=BPF\n"
                 "# NITPICKER_CAPMETHOD=PCAPLIVE\n"
                 "NITPICKER_CAPMETHOD=PCAPFILE\n" );
         break;

     default:
         printf( "# NITPICKER_CAPMETHOD=BPF\n"
                 "# NITPICKER_CAPMETHOD=PCAPLIVE\n"
                 "# NITPICKER_CAPMETHOD=PCAPFILE\n" );
         break;
 }

 printf("NITPICKER_CAPDEVICE=%s\n", cfg->capture_device );
 printf("NITPICKER_SAVEDIR=%s\n", cfg->savedir );
 printf("NITPICKER_HASHSIZE=%lu\n", cfg->hashsize );
 printf("NITPICKER_MAXFLOWSINQUEUE=%lu\n", cfg->max_flows_in_queue );
 printf("NITPICKER_FLOWSPERFILE=%lu\n", cfg->flows_per_file );
 printf("NITPICKER_FLOWTIMEOUT=%lu\n", cfg->flow_timeout ); 
 printf("NITPICKER_STATDELAY=%lu\n", cfg->stat_delay ); 
 printf("NITPICKER_BPFSIZE=%lu\n", cfg->bpf_size );
 printf("NITPICKER_BPFTIMEOUT=%lu\n", cfg->bpf_timeout );

 exit(1);
}

