#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <pcap.h>
#include <unistd.h>

#include "config.h"
#include "iphandler.h"
#include "flowsaver.h"
#include "bpf.h"
#include "pcap.h"
#include "eth.h"
#include "stats.h"
#include "nitpicker.h"
#include "mysignal.h"
#include "stats.h"
#include "version.h"


// static int(*readloop)(void);
struct config* cfg;

void write_pidfile( void );
int kerl( void );

int main( int argc, char **argv )

{
    cfg = config_init( argc, argv );

    printf("N.Starting Nitpicker:%s\n",Version);

    write_pidfile();

    /* init the handlers */
    signalhandler_init();
    iphandler_init( cfg->hashsize, cfg->max_flows_in_queue, cfg->flow_timeout );
    flowsaver_init( cfg->savedir, cfg->flows_per_file );

    init_stats( cfg->stat_delay );
    kerl();

    iphandler_check( 0x7fffffff );
    flowsaver_flush();
    dump_stats();

    if (cfg->pidfile) unlink(cfg->pidfile);

    return 0;
}


void write_pidfile( void )
{
    int pidfd;
    char strpid[10];

    if( cfg->pidfile[0] == '\0' ) return;

    printf( "N:Using %s for pidfile\n", cfg->pidfile ); 

    pidfd = open( cfg->pidfile, O_RDWR|O_CREAT|O_TRUNC, S_IRWXU|S_IRWXG ); 

    if( pidfd < 0 ) {
        perror( "while opening pidfile" );
        exit(1);
    }

    sprintf( strpid, "%u", getpid() );
    write( pidfd, strpid, sizeof strpid );

    close( pidfd );
}

void remove_pidfile( void ) {
    unlink( cfg->pidfile );
}

int kerl()
{
    char errbuf[256];
    pcap_t *pcap; 

    switch( cfg->capture_method ) {

        case CM_BPF:
            bpf_init( cfg->bpf_size, cfg->bpf_timeout );
            bpf_open( cfg->capture_device );
            bpf_readloop(); 
            break;

        case CM_PCAPLIVE: 
            pcap = pcap_open_live( cfg->capture_device, 256, 1, 1000, errbuf );
            pcap_readloop(pcap);
            break;

        case CM_PCAPFILE:
            pcap = pcap_open_offline( cfg->capture_device, errbuf ); 
            pcap_readloop(pcap);
            break; 

        default:
            printf( "E: no capture method defined.\n" );
            config_usage();
            exit(1);
            break;
    }

    return(0);
}
