#include <sys/types.h>
#include <sys/time.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <net/if.h>
#include <unistd.h>
#include "config.h"
#include "pcap.h"
#include "stats.h"
#include "iphandler.h"
#include "nitpicker.h"
#include "flowsaver.h"

/* 
 * Directly substitute libpcap with 
 * Berkeley Packet Filter
 * pcap is only able to handel 32k Buffersize.
 *
 */

static int ph_len;
static uint32 ts;
static void (*packethandler)(char *, uint32);

void pcap_packet( u_char *user, const struct pcap_pkthdr *h, const u_char *p )
{
  ts = h->ts.tv_sec;
  packethandler( (char*)p+ph_len,ts);
}


int pcap_readloop(pcap_t *pcap)
{
  int p_read;
  int oldts=0;
  int datalink;
  u_char *data; 
  struct timeval mytv;
  struct pcap_stat ps;

  datalink = pcap_datalink(pcap); 

  switch( datalink ) {

    case DLT_EN10MB:
      ph_len = 0;               /* We don't know what kind of packet will come v
ia Ethernet */
      packethandler = parse_etherpkt;  /* Create Functionpointer */
      break;

    case DLT_NULL:
      ph_len = 4;
      packethandler = iphandler_addPacket; /* Create another Functionpointer */
      break;
  }


  for( ;; )
  {
    p_read = pcap_dispatch(pcap,100,pcap_packet,data);

    if ( p_read < 0 )
    {
      pcap_perror(pcap,"read failed\n");
      exit(1);
    }
   
    if (!p_read) 
    { 

      if ( pcap_file(pcap) ) {
        break;
      }

      gettimeofday(&mytv,0);
      ts = mytv.tv_sec;
    } 
    pcap_stats(pcap, &ps);

    stats.total_pktcnt=ps.ps_recv;
    stats.total_dropped=ps.ps_drop;

    if (is_stats_time(ts))  dump_stats();

    if(  ts - oldts > 5 )  {
      oldts = ts;
      iphandler_check( ts );
    }
  }

  return 0;
}
