#include <stdio.h>
#include <fcntl.h>
#include <sys/param.h>
#include <unistd.h>
#include <string.h>
#include <libgen.h>
#include "types.h"      
#include "convert.h"


char *buildip (uint32 ipaddr)
{
  static char b[18];
  unsigned int i1,i2,i3,i4;
  
  i1 = ipaddr >> 24 ;
  i2 = (ipaddr >> 16 ) & 0xff;
  i3 = (ipaddr >> 8) & 0xff;
  i4 = ipaddr & 0xff;
  
  sprintf(b,"%u.%u.%u.%u",i1,i2,i3,i4);
  return (b);
}

int main( int argc, char* argv[] )
{
    int infile, outfile, retval;
    struct v1header v1head;
    struct v1tcpflow v1tcpf;
    struct v1othflow v1othf;
    struct v3header v3head; 
    struct v3flow   v3flw; 
//    struct v4flow   v4flw; 
    u_char flowtype=0; /* tcp, udp, other ... */
    char outfilename[22];


    printf("%d\n",sizeof(struct v4flow));
    printf("%d\n",sizeof(struct v3flow));
    exit(1);

    if( argc < 2 ) {
        printf( "%s <inputfile> \n", argv[0] );
        exit( 1 );
    }

    infile = open( argv[1], O_RDONLY );

    if( ! infile ) {
        perror( "while opening input file" );
        exit( 1 );
    }

    if      (0==strncmp(basename(argv[1]),"tcp.",4) ) flowtype=17;
    else if (0==strncmp(basename(argv[1]),"udp.",4) ) flowtype=6;
    else if (0==strncmp(basename(argv[1]),"other..",6) ) flowtype=1; /* We should look insiede */
    else {
       printf("Unknown Flow:%s\n",argv[1]);
       exit(1);
    }

    printf("%s is %d\n",basename(argv[1]), flowtype );

    retval = read( infile, &v1head, sizeof( struct v1header ) );

    if( !retval ) {
        perror( "while reading header from input file" );
        exit(1);
    }

    sprintf(outfilename,"flow.%lu.s",v1head.starttime);
    printf("Using %s as outfile\n", outfilename);
    outfile = open( outfilename , O_RDWR | O_CREAT | O_TRUNC, 0666 );

    if( ! outfile ) {
        perror( "while opening output file" );
        exit (1);
    }


    printf( "starttime:%lu  stoptime:%lu nbytes:%qu nflows:%lu npkts:%lu\n",
            v1head.starttime,
            v1head.stoptime,
            v1head.nbytes,
            v1head.nflows,
            v1head.npkts );
    
    bzero(&v3head,sizeof( struct v3header) ); 
    retval = write( outfile, &v3head, sizeof( struct v3header ) );
    if (!retval) {
       perror("Write outfile header\n");
       exit(1);
    }

    switch(flowtype) { /* 17,6,1 */
    
      case  6:
      case 17:
     
        for(;( retval = read( infile, &v1tcpf, sizeof( struct v1tcpflow ) ) ); ) {
           v3flw.startoffset = htons(v1tcpf.startoffset);
           v3flw.endoffset = htons(v1tcpf.endoffset);
           v3flw.sdpkts = htonl(v1tcpf.pkts) ;
           v3flw.sdbytes = htonl(v1tcpf.bytes) ;
           v3flw.dspkts = 0 ;
           v3flw.dsbytes = 0 ;

           v3flw.src = v1tcpf.src;  
           v3flw.dst =  v1tcpf.dst ;
           v3flw.typenr = flowtype;
           v3flw.tcp.sport = v1tcpf.sport;
           v3flw.tcp.dport = v1tcpf.dport;
      
           write(outfile,&v3flw,sizeof(struct v3flow) );
 
           v3head.starttime=htonl(v1head.starttime);
           if(v3head.stoptime < v3head.starttime+v1tcpf.startoffset)
              v3head.stoptime=htonl(v3head.starttime+v1tcpf.startoffset);
 
           v3head.bytes=v3head.bytes + v1tcpf.bytes;
           v3head.flows++;
           v3head.pkts=v3head.pkts+v1tcpf.pkts;
        }
        break;

    case 1:
       
        for(;( retval = read( infile, &v1othf, sizeof( struct v1othflow ) ) ); ) {
           v3flw.startoffset = htons(v1othf.startoffset);
           v3flw.endoffset = htons(v1othf.endoffset);
           v3flw.sdpkts = htonl(v1othf.pkts) ;
           v3flw.sdbytes = htonl(v1othf.bytes) ;
           v3flw.dspkts = 0 ;
           v3flw.dsbytes = 0 ;

           v3flw.src = v1othf.src; 
           v3flw.dst =  v1othf.dst ;
           v3flw.typenr = v1othf.type;
        
           write(outfile,&v3flw,sizeof(struct v3flow) );

           v3head.starttime=htonl(v1head.starttime);
           if(v3head.stoptime < v3head.starttime+v1othf.startoffset)
              v3head.stoptime=htonl(v3head.starttime+v1othf.startoffset);
            
           v3head.bytes=v3head.bytes + v1othf.bytes;
           v3head.flows++;
           v3head.pkts=v3head.pkts+v1othf.pkts;
        }
        break;

     
    }

    

    close( infile );
    
    strncpy(v3head.MagicNr, "NPFF",4 );

    v3head.MajVersion = 3;
    v3head.MinVersion = 1;

    lseek( outfile, 0, SEEK_SET );  
    write( outfile, &v3head, sizeof( struct v3header ) );

    close( outfile );
    
    return 0;
}

