#include <stdio.h>
#include <fcntl.h>
#include <sys/param.h>
#include <unistd.h>
#include <string.h>
#include <libgen.h>
#include "convert.h"
#include "types.h"


char *buildip (uint32 ipaddr)
{
  static char b[18];
  unsigned int i1,i2,i3,i4;
  
  i1 = ipaddr >> 24 ;
  i2 = (ipaddr >> 16 ) & 0xff;
  i3 = (ipaddr >> 8) & 0xff;
  i4 = ipaddr & 0xff;
  
  sprintf(b,"%u.%u.%u.%u",i1,i2,i3,i4);
  return (b);
}

int main( int argc, char* argv[] )
{
    int infile, outfile, retval;

    struct v4header v4head; 
    struct v4flow   v4flw; 

    struct v2header v2head;
    struct v2flow   v2flw;
    struct v2tcpep  tcpep;
    struct v2udpep  udpep;

    u_char flowtype=0; /* tcp, udp, other ... */
    char outfilename[22];
    char newfilename[22];


    if( argc < 2 ) {
        printf( "%s <inputfile> \n", argv[0] );
        exit( 1 );
    }

    infile = open( argv[1], O_RDONLY );

    if( ! infile ) {
        perror( "while opening input file" );
        exit( 1 );
    }

    printf("%s is %d\n",basename(argv[1]), flowtype );

    retval = read( infile, &v2head, sizeof( struct v2header ) );

    if( !retval ) {
        perror( "while reading header from input file" );
        exit(1);
    }

    printf( "starttime:%lu  stoptime:%lu nbytes:%qu nflows:%lu npkts:%lu\n",
            v2head.starttime,
            v2head.stoptime,
            v2head.bytes,
            v2head.flows,
            v2head.pkts 
          );


    sprintf(outfilename,"flow.%lu.u",v2head.starttime);
    sprintf(newfilename,"flow.%lu.s",v2head.starttime);

    printf("Using %s as outfile\n", outfilename);
    outfile = open( outfilename , O_RDWR | O_CREAT | O_TRUNC, 0666 );

    if( ! outfile ) {
        perror( "while opening output file" );
        exit (1);
    }


    bzero(&v4head,sizeof( struct v4header) ); 
    retval = write( outfile, &v4head, sizeof( struct v4header ) );
    if (!retval) {
       perror("Write outfile header\n");
       exit(1);
    }

    v4head.starttime = htonl (v2head.starttime);
    v4head.stoptime  = htonl (v2head.stoptime);
    v4head.bytes     = htonq (v2head.bytes);
    v4head.flows     = htonl (v2head.flows);
    v4head.pkts      = htonl (v2head.pkts);

    for(;( retval = read ( infile, &v2flw, sizeof( struct v2flow) ) ) ; ) {
 
        if (v2flw.typenr == 17 ) {
            read( infile, &udpep, sizeof( struct v2udpep) );
            v4flw.udp.sport = (udpep.sport);
            v4flw.udp.dport = (udpep.dport);
        }

        if (v2flw.typenr ==  6 ) {
           read( infile, &tcpep, sizeof( struct v2tcpep) );
           v4flw.tcp.sport =  (tcpep.sport);
           v4flw.tcp.dport =  (tcpep.dport);
        }

        v4flw.typenr      = v2flw.typenr;

        v4flw.startoffset = htons(v2flw.startoffset);
        v4flw.endoffset   = htons(v2flw.endoffset);
        v4flw.sdpkts      = htonl(v2flw.sdpkts) ;
        v4flw.dspkts      = htonl(v2flw.dspkts);
        v4flw.sdbytes     = htonl(v2flw.sdbytes) ;
        v4flw.dsbytes     = htonl(v2flw.dsbytes) ;
        v4flw.src         = (v2flw.src) ;  
        v4flw.dst         = (v2flw.dst) ;  
      
        if( sizeof(struct v4flow) != write(outfile,&v4flw,sizeof(struct v4flow) ) ) {
           perror("write");
           exit(1);
        }
 
    }


    close( infile );
    
    v4head.MagicNr[0]='N';
    v4head.MagicNr[1]='P'; 
    v4head.MagicNr[2]='F';
    v4head.MagicNr[3]='F';

    v4head.MajVersion = 4;
    v4head.MinVersion = 0;

    lseek( outfile, 0, SEEK_SET );  
    write( outfile, &v4head, sizeof( struct v4header ) );

    close( outfile );

    rename (outfilename,newfilename );                         
    
    return 0;
}

