#include <stdio.h>
#include <fcntl.h>
#include <sys/param.h>
#include <unistd.h>
#include <string.h>
#include "types.h"
#include "convert.h"


char *buildip (uint32 ipaddr)
{
  static char b[18];
  unsigned int i1,i2,i3,i4;
  
  i1 = ipaddr >> 24 ;
  i2 = (ipaddr >> 16 ) & 0xff;
  i3 = (ipaddr >> 8) & 0xff;
  i4 = ipaddr & 0xff;
  
  sprintf(b,"%u.%u.%u.%u",i1,i2,i3,i4);
  return (b);
}

int main( int argc, char* argv[] )
{
    int infile, retval;

    struct v2flow   v2flw;
    struct v2header v2head;
    struct v2tcpep  tcpep;
    struct v2udpep  udpep;

    if( argc < 2 ) {
        printf( "%s <inputfile> ", argv[0] );
        exit( 1 );
    }

    infile = open( argv[1], O_RDONLY );
    if( ! infile ) {
        perror( "while opening input file" );
        exit( 1 );
    }

    retval = read( infile, &v2head, sizeof( struct v2header ) );
    if( !retval ) {
        perror( "while reading header from input file" );
        exit(1);
    }

    printf( "starttime:%lu  stoptime:%lu nbytes:%qu nflows:%lu npkts:%lu\n",
            v2head.starttime,
            v2head.stoptime,
            v2head.bytes,
            v2head.flows,
            v2head.pkts 
          );


    for(;( retval = read( infile, &v2flw, sizeof( struct v2flow ) ) ); ) {
 
        if (v2flw.typenr == 17 )  read( infile, &udpep, sizeof( struct v2udpep) );
        if (v2flw.typenr ==  6 )  read( infile, &tcpep, sizeof( struct v2tcpep) );

        printf(  "proto:%d start:%d stop:%d %s",
                v2flw.typenr,
                (signed short) v2flw.startoffset,
                (signed short) v2flw.endoffset,
                buildip( ntohl(v2flw.src) )
              );

       if ( v2flw.typenr==17) printf(":%u ",htons( udpep.dport) );
       if ( v2flw.typenr== 6) printf(":%u ",htons( tcpep.dport) );


        printf( " %s", 
                buildip( ntohl(v2flw.dst) )
              );

       if ( v2flw.typenr==17) printf(":%u ",htons( udpep.sport) );
       if ( v2flw.typenr== 6) printf(":%u ",htons( tcpep.sport) );



        printf( " %lu %lu %lu %lu\n",
                (v2flw.sdpkts),
                (v2flw.dspkts),
                (v2flw.sdbytes),
                (v2flw.dsbytes)
             );

    }

    close( infile );
    return 0;
}

