#include <stdio.h>
#include <fcntl.h>
#include <sys/param.h>
#include <unistd.h>
#include <string.h>
#include "types.h"       
#include "convert.h"


char *buildip (uint32 ipaddr)
{
  static char b[18];
  unsigned int i1,i2,i3,i4;
  
  i1 = ipaddr >> 24 ;
  i2 = (ipaddr >> 16 ) & 0xff;
  i3 = (ipaddr >> 8) & 0xff;
  i4 = ipaddr & 0xff;
  
  sprintf(b,"%u.%u.%u.%u",i1,i2,i3,i4);
  return (b);
}

int main( int argc, char* argv[] )
{
    int infile, retval;
    struct v3flow   v3flw;
    struct v3header v3head; 

    if( argc < 2 ) {
        printf( "%s <inputfile> ", argv[0] );
        exit( 1 );
    }

    infile = open( argv[1], O_RDONLY );
    if( ! infile ) {
        perror( "while opening input file" );
        exit( 1 );
    }

    retval = read( infile, &v3head, sizeof( struct v3header ) );
    if( !retval ) {
        perror( "while reading header from input file" );
        exit(1);
    }

    printf( "Magic:%4s Version:%d.%d\n", 
             v3head.MagicNr,
             v3head.MajVersion,
             v3head.MinVersion
           );

    printf( "starttime:%lu  stoptime:%lu nbytes:%qu nflows:%lu npkts:%lu\n",
            ntohl(v3head.starttime),
            ntohl(v3head.stoptime),
            v3head.bytes,
            v3head.flows,
            v3head.pkts 
          );


    for(;( retval = read( infile, &v3flw, sizeof( struct v3flow ) ) ); ) {
        printf(  "proto:%d start:%d stop:%d %s ",
                v3flw.typenr,
                (signed short) ntohs(v3flw.startoffset),
                (signed short) ntohs(v3flw.endoffset),
                buildip( htonl( v3flw.src ) )
              );
        if ( (v3flw.typenr==17) || (v3flw.typenr==6) )
           printf(":%u ",htons(v3flw.tcp.sport) );

        printf( " %s ", 
                buildip( htonl( v3flw.dst ) )
              );
        if ( (v3flw.typenr==17) || (v3flw.typenr==6) )
           printf(":%u ",htons( v3flw.tcp.dport) );

        printf( "%lu %lu %lu %lu\n",
                htonl(v3flw.sdpkts),
                htonl(v3flw.dspkts),
                htonl(v3flw.sdbytes),
                htonl(v3flw.dsbytes)
             );

    }

    close( infile );
    return 0;
}

