#include "types.h"   
#include <sys/types.h>
#include <machine/endian.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/param.h>
#include <unistd.h>
#include <string.h>
#include "convert.h"


char *buildip (uint32 ipaddr)
{
  static char b[18];
  unsigned int i1,i2,i3,i4;
  
  i1 = ipaddr >> 24 ;
  i2 = (ipaddr >> 16 ) & 0xff;
  i3 = (ipaddr >> 8) & 0xff;
  i4 = ipaddr & 0xff;
  
  sprintf(b,"%u.%u.%u.%u",i1,i2,i3,i4);
  return (b);
}

int main( int argc, char* argv[] )
{
    int infile, retval;
    struct v4flow   v4flw;
    struct v4header v4head; 

    if( argc < 2 ) {
        printf( "%s <inputfile> ", argv[0] );
        exit( 1 );
    }

    infile = open( argv[1], O_RDONLY );
    if( ! infile ) {
        perror( "while opening input file" );
        exit( 1 );
    }

    retval = read( infile, &v4head, sizeof( struct v4header ) );
    if( !retval ) {
        perror( "while reading header from input file" );
        exit(1);
    }

    

    if (4 != v4head.MajVersion ) {
       printf("No V4 Flowfile\n");
       exit(1);
    }

    printf( "Magic:%4s Version:%d.%d\n", 
             v4head.MagicNr,
             v4head.MajVersion,
             v4head.MinVersion
           );

    printf( "starttime:%u  stoptime:%u nbytes:%qu nflows:%u npkts:%u\n",
            ntohl (v4head.starttime),
            ntohl (v4head.stoptime),
            ntohq (v4head.bytes),
            ntohl (v4head.flows),
            ntohl (v4head.pkts) 
          );


    for(;( retval = read( infile, &v4flw, sizeof( struct v4flow ) ) ); ) {
        printf(  "proto:%d start:%d stop:%d %s",
                v4flw.typenr,
                (signed short) ntohs(v4flw.startoffset),
                (signed short) ntohs(v4flw.endoffset),
                buildip( ntohl( v4flw.src ) )
              );

        if      (v4flw.typenr==17) printf(":%u",ntohs(v4flw.tcp.sport) );
        else if (v4flw.typenr== 6) printf(":%u",ntohs(v4flw.udp.sport) );

        printf( " %s", buildip( ntohl( v4flw.dst ) ));

        if      (v4flw.typenr==17) printf(":%u",ntohs(v4flw.tcp.dport) );
        else if (v4flw.typenr== 6) printf(":%u",ntohs(v4flw.udp.dport) );

        printf( " %u %u %u %u\n",
                htonl(v4flw.sdpkts),
                htonl(v4flw.dspkts),
                htonl(v4flw.sdbytes),
                htonl(v4flw.dsbytes)
             );

    }

    close( infile );
    return 0;
}

